"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
import httpx
import pydantic
from pydantic import model_serializer
from typing import Any, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class FieldCreateDocumentFieldTypeDropdownRequest1(str, Enum):
    DROPDOWN = "DROPDOWN"


class FieldCreateDocumentFieldTypeDropdownRequest2(str, Enum):
    DROPDOWN = "dropdown"


class FieldCreateDocumentFieldValueDropdownTypedDict(TypedDict):
    value: str


class FieldCreateDocumentFieldValueDropdown(BaseModel):
    value: str


class FieldCreateDocumentFieldFieldMetaDropdownRequestTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeDropdownRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldCreateDocumentFieldValueDropdownTypedDict]]
    default_value: NotRequired[str]


class FieldCreateDocumentFieldFieldMetaDropdownRequest(BaseModel):
    type: FieldCreateDocumentFieldTypeDropdownRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldCreateDocumentFieldValueDropdown]] = None

    default_value: Annotated[Optional[str], pydantic.Field(alias="defaultValue")] = None


class FieldCreateDocumentFieldFieldDropdownTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeDropdownRequest1
    recipient_id: float
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[FieldCreateDocumentFieldFieldMetaDropdownRequestTypedDict]


class FieldCreateDocumentFieldFieldDropdown(BaseModel):
    type: FieldCreateDocumentFieldTypeDropdownRequest1

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[FieldCreateDocumentFieldFieldMetaDropdownRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None


class FieldCreateDocumentFieldTypeCheckboxRequest1(str, Enum):
    CHECKBOX = "CHECKBOX"


class FieldCreateDocumentFieldTypeCheckboxRequest2(str, Enum):
    CHECKBOX = "checkbox"


class FieldCreateDocumentFieldValueCheckboxTypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldCreateDocumentFieldValueCheckbox(BaseModel):
    id: float

    checked: bool

    value: str


class FieldCreateDocumentFieldDirectionCheckbox(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldCreateDocumentFieldFieldMetaCheckboxRequestTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeCheckboxRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldCreateDocumentFieldValueCheckboxTypedDict]]
    validation_rule: NotRequired[str]
    validation_length: NotRequired[float]
    direction: NotRequired[FieldCreateDocumentFieldDirectionCheckbox]


class FieldCreateDocumentFieldFieldMetaCheckboxRequest(BaseModel):
    type: FieldCreateDocumentFieldTypeCheckboxRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldCreateDocumentFieldValueCheckbox]] = None

    validation_rule: Annotated[
        Optional[str], pydantic.Field(alias="validationRule")
    ] = None

    validation_length: Annotated[
        Optional[float], pydantic.Field(alias="validationLength")
    ] = None

    direction: Optional[FieldCreateDocumentFieldDirectionCheckbox] = (
        FieldCreateDocumentFieldDirectionCheckbox.VERTICAL
    )


class FieldCreateDocumentFieldFieldCheckboxTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeCheckboxRequest1
    recipient_id: float
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[FieldCreateDocumentFieldFieldMetaCheckboxRequestTypedDict]


class FieldCreateDocumentFieldFieldCheckbox(BaseModel):
    type: FieldCreateDocumentFieldTypeCheckboxRequest1

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[FieldCreateDocumentFieldFieldMetaCheckboxRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None


class FieldCreateDocumentFieldTypeRadioRequest1(str, Enum):
    RADIO = "RADIO"


class FieldCreateDocumentFieldTypeRadioRequest2(str, Enum):
    RADIO = "radio"


class FieldCreateDocumentFieldValueRadioTypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldCreateDocumentFieldValueRadio(BaseModel):
    id: float

    checked: bool

    value: str


class FieldCreateDocumentFieldDirectionRadio(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldCreateDocumentFieldFieldMetaRadioRequestTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeRadioRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldCreateDocumentFieldValueRadioTypedDict]]
    direction: NotRequired[FieldCreateDocumentFieldDirectionRadio]


class FieldCreateDocumentFieldFieldMetaRadioRequest(BaseModel):
    type: FieldCreateDocumentFieldTypeRadioRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldCreateDocumentFieldValueRadio]] = None

    direction: Optional[FieldCreateDocumentFieldDirectionRadio] = (
        FieldCreateDocumentFieldDirectionRadio.VERTICAL
    )


class FieldCreateDocumentFieldFieldRadioTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeRadioRequest1
    recipient_id: float
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[FieldCreateDocumentFieldFieldMetaRadioRequestTypedDict]


class FieldCreateDocumentFieldFieldRadio(BaseModel):
    type: FieldCreateDocumentFieldTypeRadioRequest1

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[FieldCreateDocumentFieldFieldMetaRadioRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None


class FieldCreateDocumentFieldTypeNumberRequest1(str, Enum):
    NUMBER = "NUMBER"


class FieldCreateDocumentFieldTypeNumberRequest2(str, Enum):
    NUMBER = "number"


class FieldCreateDocumentFieldTextAlignNumber(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldVerticalAlignNumber(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldCreateDocumentFieldFieldMetaNumberRequestTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeNumberRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    number_format: NotRequired[Nullable[str]]
    value: NotRequired[str]
    min_value: NotRequired[Nullable[float]]
    max_value: NotRequired[Nullable[float]]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignNumber]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[FieldCreateDocumentFieldVerticalAlignNumber]]


class FieldCreateDocumentFieldFieldMetaNumberRequest(BaseModel):
    type: FieldCreateDocumentFieldTypeNumberRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    number_format: Annotated[
        OptionalNullable[str], pydantic.Field(alias="numberFormat")
    ] = UNSET

    value: Optional[str] = None

    min_value: Annotated[OptionalNullable[float], pydantic.Field(alias="minValue")] = (
        UNSET
    )

    max_value: Annotated[OptionalNullable[float], pydantic.Field(alias="maxValue")] = (
        UNSET
    )

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignNumber],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldCreateDocumentFieldVerticalAlignNumber],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "numberFormat",
            "value",
            "minValue",
            "maxValue",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = [
            "numberFormat",
            "minValue",
            "maxValue",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldCreateDocumentFieldFieldNumberTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeNumberRequest1
    recipient_id: float
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[FieldCreateDocumentFieldFieldMetaNumberRequestTypedDict]


class FieldCreateDocumentFieldFieldNumber(BaseModel):
    type: FieldCreateDocumentFieldTypeNumberRequest1

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[FieldCreateDocumentFieldFieldMetaNumberRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None


class FieldCreateDocumentFieldTypeTextRequest1(str, Enum):
    TEXT = "TEXT"


class FieldCreateDocumentFieldTypeTextRequest2(str, Enum):
    TEXT = "text"


class FieldCreateDocumentFieldTextAlignText(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldVerticalAlignText(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldCreateDocumentFieldFieldMetaTextRequestTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeTextRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text: NotRequired[str]
    character_limit: NotRequired[float]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignText]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[FieldCreateDocumentFieldVerticalAlignText]]


class FieldCreateDocumentFieldFieldMetaTextRequest(BaseModel):
    type: FieldCreateDocumentFieldTypeTextRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text: Optional[str] = None

    character_limit: Annotated[
        Optional[float], pydantic.Field(alias="characterLimit")
    ] = None

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignText],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldCreateDocumentFieldVerticalAlignText],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "text",
            "characterLimit",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = ["lineHeight", "letterSpacing", "verticalAlign"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldCreateDocumentFieldFieldTextTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeTextRequest1
    recipient_id: float
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[FieldCreateDocumentFieldFieldMetaTextRequestTypedDict]


class FieldCreateDocumentFieldFieldText(BaseModel):
    type: FieldCreateDocumentFieldTypeTextRequest1

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[FieldCreateDocumentFieldFieldMetaTextRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None


class FieldCreateDocumentFieldTypeDateRequest1(str, Enum):
    DATE = "DATE"


class FieldCreateDocumentFieldTypeDateRequest2(str, Enum):
    DATE = "date"


class FieldCreateDocumentFieldTextAlignDate(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldFieldMetaDateRequestTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeDateRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignDate]


class FieldCreateDocumentFieldFieldMetaDateRequest(BaseModel):
    type: FieldCreateDocumentFieldTypeDateRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignDate],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldCreateDocumentFieldFieldDateTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeDateRequest1
    recipient_id: float
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[FieldCreateDocumentFieldFieldMetaDateRequestTypedDict]


class FieldCreateDocumentFieldFieldDate(BaseModel):
    type: FieldCreateDocumentFieldTypeDateRequest1

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[FieldCreateDocumentFieldFieldMetaDateRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None


class FieldCreateDocumentFieldTypeEmailRequest1(str, Enum):
    EMAIL = "EMAIL"


class FieldCreateDocumentFieldTypeEmailRequest2(str, Enum):
    EMAIL = "email"


class FieldCreateDocumentFieldTextAlignEmail(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldFieldMetaEmailRequestTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeEmailRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignEmail]


class FieldCreateDocumentFieldFieldMetaEmailRequest(BaseModel):
    type: FieldCreateDocumentFieldTypeEmailRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignEmail],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldCreateDocumentFieldFieldEmailTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeEmailRequest1
    recipient_id: float
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[FieldCreateDocumentFieldFieldMetaEmailRequestTypedDict]


class FieldCreateDocumentFieldFieldEmail(BaseModel):
    type: FieldCreateDocumentFieldTypeEmailRequest1

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[FieldCreateDocumentFieldFieldMetaEmailRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None


class FieldCreateDocumentFieldTypeNameRequest1(str, Enum):
    NAME = "NAME"


class FieldCreateDocumentFieldTypeNameRequest2(str, Enum):
    NAME = "name"


class FieldCreateDocumentFieldTextAlignName(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldFieldMetaNameRequestTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeNameRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignName]


class FieldCreateDocumentFieldFieldMetaNameRequest(BaseModel):
    type: FieldCreateDocumentFieldTypeNameRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignName],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldCreateDocumentFieldFieldNameTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeNameRequest1
    recipient_id: float
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[FieldCreateDocumentFieldFieldMetaNameRequestTypedDict]


class FieldCreateDocumentFieldFieldName(BaseModel):
    type: FieldCreateDocumentFieldTypeNameRequest1

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[FieldCreateDocumentFieldFieldMetaNameRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None


class FieldCreateDocumentFieldTypeInitialsRequest1(str, Enum):
    INITIALS = "INITIALS"


class FieldCreateDocumentFieldTypeInitialsRequest2(str, Enum):
    INITIALS = "initials"


class FieldCreateDocumentFieldTextAlignInitials(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldFieldMetaInitialsRequestTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeInitialsRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignInitials]


class FieldCreateDocumentFieldFieldMetaInitialsRequest(BaseModel):
    type: FieldCreateDocumentFieldTypeInitialsRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignInitials],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldCreateDocumentFieldFieldInitialsTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeInitialsRequest1
    recipient_id: float
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[FieldCreateDocumentFieldFieldMetaInitialsRequestTypedDict]


class FieldCreateDocumentFieldFieldInitials(BaseModel):
    type: FieldCreateDocumentFieldTypeInitialsRequest1

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[FieldCreateDocumentFieldFieldMetaInitialsRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None


class FieldCreateDocumentFieldTypeFreeSignature(str, Enum):
    FREE_SIGNATURE = "FREE_SIGNATURE"


class FieldCreateDocumentFieldFieldFreeSignatureTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeFreeSignature
    recipient_id: float
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float


class FieldCreateDocumentFieldFieldFreeSignature(BaseModel):
    type: FieldCreateDocumentFieldTypeFreeSignature

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float


class FieldCreateDocumentFieldTypeSignatureRequest1(str, Enum):
    SIGNATURE = "SIGNATURE"


class FieldCreateDocumentFieldTypeSignatureRequest2(str, Enum):
    SIGNATURE = "signature"


class FieldCreateDocumentFieldFieldMetaSignatureRequestTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeSignatureRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]


class FieldCreateDocumentFieldFieldMetaSignatureRequest(BaseModel):
    type: FieldCreateDocumentFieldTypeSignatureRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12


class FieldCreateDocumentFieldFieldSignatureTypedDict(TypedDict):
    type: FieldCreateDocumentFieldTypeSignatureRequest1
    recipient_id: float
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[FieldCreateDocumentFieldFieldMetaSignatureRequestTypedDict]


class FieldCreateDocumentFieldFieldSignature(BaseModel):
    type: FieldCreateDocumentFieldTypeSignatureRequest1

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[FieldCreateDocumentFieldFieldMetaSignatureRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None


FieldCreateDocumentFieldFieldUnionTypedDict = TypeAliasType(
    "FieldCreateDocumentFieldFieldUnionTypedDict",
    Union[
        FieldCreateDocumentFieldFieldFreeSignatureTypedDict,
        FieldCreateDocumentFieldFieldSignatureTypedDict,
        FieldCreateDocumentFieldFieldInitialsTypedDict,
        FieldCreateDocumentFieldFieldNameTypedDict,
        FieldCreateDocumentFieldFieldEmailTypedDict,
        FieldCreateDocumentFieldFieldDateTypedDict,
        FieldCreateDocumentFieldFieldTextTypedDict,
        FieldCreateDocumentFieldFieldNumberTypedDict,
        FieldCreateDocumentFieldFieldRadioTypedDict,
        FieldCreateDocumentFieldFieldCheckboxTypedDict,
        FieldCreateDocumentFieldFieldDropdownTypedDict,
    ],
)


FieldCreateDocumentFieldFieldUnion = TypeAliasType(
    "FieldCreateDocumentFieldFieldUnion",
    Union[
        FieldCreateDocumentFieldFieldFreeSignature,
        FieldCreateDocumentFieldFieldSignature,
        FieldCreateDocumentFieldFieldInitials,
        FieldCreateDocumentFieldFieldName,
        FieldCreateDocumentFieldFieldEmail,
        FieldCreateDocumentFieldFieldDate,
        FieldCreateDocumentFieldFieldText,
        FieldCreateDocumentFieldFieldNumber,
        FieldCreateDocumentFieldFieldRadio,
        FieldCreateDocumentFieldFieldCheckbox,
        FieldCreateDocumentFieldFieldDropdown,
    ],
)


class FieldCreateDocumentFieldRequestTypedDict(TypedDict):
    document_id: float
    field: FieldCreateDocumentFieldFieldUnionTypedDict


class FieldCreateDocumentFieldRequest(BaseModel):
    document_id: Annotated[float, pydantic.Field(alias="documentId")]

    field: FieldCreateDocumentFieldFieldUnion


class FieldCreateDocumentFieldInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class FieldCreateDocumentFieldInternalServerErrorIssue(BaseModel):
    message: str


class FieldCreateDocumentFieldInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldCreateDocumentFieldInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class FieldCreateDocumentFieldInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: FieldCreateDocumentFieldInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldCreateDocumentFieldInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldCreateDocumentFieldForbiddenIssueTypedDict(TypedDict):
    message: str


class FieldCreateDocumentFieldForbiddenIssue(BaseModel):
    message: str


class FieldCreateDocumentFieldForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldCreateDocumentFieldForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class FieldCreateDocumentFieldForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: FieldCreateDocumentFieldForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldCreateDocumentFieldForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldCreateDocumentFieldUnauthorizedIssueTypedDict(TypedDict):
    message: str


class FieldCreateDocumentFieldUnauthorizedIssue(BaseModel):
    message: str


class FieldCreateDocumentFieldUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldCreateDocumentFieldUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class FieldCreateDocumentFieldUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: FieldCreateDocumentFieldUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldCreateDocumentFieldUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldCreateDocumentFieldBadRequestIssueTypedDict(TypedDict):
    message: str


class FieldCreateDocumentFieldBadRequestIssue(BaseModel):
    message: str


class FieldCreateDocumentFieldBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldCreateDocumentFieldBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class FieldCreateDocumentFieldBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: FieldCreateDocumentFieldBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldCreateDocumentFieldBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldCreateDocumentFieldTypeResponse(str, Enum):
    SIGNATURE = "SIGNATURE"
    FREE_SIGNATURE = "FREE_SIGNATURE"
    INITIALS = "INITIALS"
    NAME = "NAME"
    EMAIL = "EMAIL"
    DATE = "DATE"
    TEXT = "TEXT"
    NUMBER = "NUMBER"
    RADIO = "RADIO"
    CHECKBOX = "CHECKBOX"
    DROPDOWN = "DROPDOWN"


class FieldCreateDocumentFieldFieldMetaTypeDropdown(str, Enum):
    DROPDOWN = "dropdown"


class FieldCreateDocumentFieldValueResponse3TypedDict(TypedDict):
    value: str


class FieldCreateDocumentFieldValueResponse3(BaseModel):
    value: str


class FieldCreateDocumentFieldFieldMetaDropdownResponseTypedDict(TypedDict):
    type: FieldCreateDocumentFieldFieldMetaTypeDropdown
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldCreateDocumentFieldValueResponse3TypedDict]]
    default_value: NotRequired[str]


class FieldCreateDocumentFieldFieldMetaDropdownResponse(BaseModel):
    type: FieldCreateDocumentFieldFieldMetaTypeDropdown

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldCreateDocumentFieldValueResponse3]] = None

    default_value: Annotated[Optional[str], pydantic.Field(alias="defaultValue")] = None


class FieldCreateDocumentFieldFieldMetaTypeCheckbox(str, Enum):
    CHECKBOX = "checkbox"


class FieldCreateDocumentFieldValueResponse2TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldCreateDocumentFieldValueResponse2(BaseModel):
    id: float

    checked: bool

    value: str


class FieldCreateDocumentFieldDirectionResponse2(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldCreateDocumentFieldFieldMetaCheckboxResponseTypedDict(TypedDict):
    type: FieldCreateDocumentFieldFieldMetaTypeCheckbox
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldCreateDocumentFieldValueResponse2TypedDict]]
    validation_rule: NotRequired[str]
    validation_length: NotRequired[float]
    direction: NotRequired[FieldCreateDocumentFieldDirectionResponse2]


class FieldCreateDocumentFieldFieldMetaCheckboxResponse(BaseModel):
    type: FieldCreateDocumentFieldFieldMetaTypeCheckbox

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldCreateDocumentFieldValueResponse2]] = None

    validation_rule: Annotated[
        Optional[str], pydantic.Field(alias="validationRule")
    ] = None

    validation_length: Annotated[
        Optional[float], pydantic.Field(alias="validationLength")
    ] = None

    direction: Optional[FieldCreateDocumentFieldDirectionResponse2] = (
        FieldCreateDocumentFieldDirectionResponse2.VERTICAL
    )


class FieldCreateDocumentFieldFieldMetaTypeRadio(str, Enum):
    RADIO = "radio"


class FieldCreateDocumentFieldValueResponse1TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldCreateDocumentFieldValueResponse1(BaseModel):
    id: float

    checked: bool

    value: str


class FieldCreateDocumentFieldDirectionResponse1(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldCreateDocumentFieldFieldMetaRadioResponseTypedDict(TypedDict):
    type: FieldCreateDocumentFieldFieldMetaTypeRadio
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldCreateDocumentFieldValueResponse1TypedDict]]
    direction: NotRequired[FieldCreateDocumentFieldDirectionResponse1]


class FieldCreateDocumentFieldFieldMetaRadioResponse(BaseModel):
    type: FieldCreateDocumentFieldFieldMetaTypeRadio

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldCreateDocumentFieldValueResponse1]] = None

    direction: Optional[FieldCreateDocumentFieldDirectionResponse1] = (
        FieldCreateDocumentFieldDirectionResponse1.VERTICAL
    )


class FieldCreateDocumentFieldFieldMetaTypeNumber(str, Enum):
    NUMBER = "number"


class FieldCreateDocumentFieldTextAlignResponse6(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldVerticalAlignResponse2(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldCreateDocumentFieldFieldMetaNumberResponseTypedDict(TypedDict):
    type: FieldCreateDocumentFieldFieldMetaTypeNumber
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    number_format: NotRequired[Nullable[str]]
    value: NotRequired[str]
    min_value: NotRequired[Nullable[float]]
    max_value: NotRequired[Nullable[float]]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignResponse6]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[
        Nullable[FieldCreateDocumentFieldVerticalAlignResponse2]
    ]


class FieldCreateDocumentFieldFieldMetaNumberResponse(BaseModel):
    type: FieldCreateDocumentFieldFieldMetaTypeNumber

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    number_format: Annotated[
        OptionalNullable[str], pydantic.Field(alias="numberFormat")
    ] = UNSET

    value: Optional[str] = None

    min_value: Annotated[OptionalNullable[float], pydantic.Field(alias="minValue")] = (
        UNSET
    )

    max_value: Annotated[OptionalNullable[float], pydantic.Field(alias="maxValue")] = (
        UNSET
    )

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignResponse6],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldCreateDocumentFieldVerticalAlignResponse2],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "numberFormat",
            "value",
            "minValue",
            "maxValue",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = [
            "numberFormat",
            "minValue",
            "maxValue",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldCreateDocumentFieldFieldMetaTypeText(str, Enum):
    TEXT = "text"


class FieldCreateDocumentFieldTextAlignResponse5(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldVerticalAlignResponse1(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldCreateDocumentFieldFieldMetaTextResponseTypedDict(TypedDict):
    type: FieldCreateDocumentFieldFieldMetaTypeText
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text: NotRequired[str]
    character_limit: NotRequired[float]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignResponse5]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[
        Nullable[FieldCreateDocumentFieldVerticalAlignResponse1]
    ]


class FieldCreateDocumentFieldFieldMetaTextResponse(BaseModel):
    type: FieldCreateDocumentFieldFieldMetaTypeText

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text: Optional[str] = None

    character_limit: Annotated[
        Optional[float], pydantic.Field(alias="characterLimit")
    ] = None

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignResponse5],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldCreateDocumentFieldVerticalAlignResponse1],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "text",
            "characterLimit",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = ["lineHeight", "letterSpacing", "verticalAlign"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldCreateDocumentFieldFieldMetaTypeDate(str, Enum):
    DATE = "date"


class FieldCreateDocumentFieldTextAlignResponse4(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldFieldMetaDateResponseTypedDict(TypedDict):
    type: FieldCreateDocumentFieldFieldMetaTypeDate
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignResponse4]


class FieldCreateDocumentFieldFieldMetaDateResponse(BaseModel):
    type: FieldCreateDocumentFieldFieldMetaTypeDate

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignResponse4],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldCreateDocumentFieldFieldMetaTypeEmail(str, Enum):
    EMAIL = "email"


class FieldCreateDocumentFieldTextAlignResponse3(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldFieldMetaEmailResponseTypedDict(TypedDict):
    type: FieldCreateDocumentFieldFieldMetaTypeEmail
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignResponse3]


class FieldCreateDocumentFieldFieldMetaEmailResponse(BaseModel):
    type: FieldCreateDocumentFieldFieldMetaTypeEmail

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignResponse3],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldCreateDocumentFieldFieldMetaTypeName(str, Enum):
    NAME = "name"


class FieldCreateDocumentFieldTextAlignResponse2(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldFieldMetaNameResponseTypedDict(TypedDict):
    type: FieldCreateDocumentFieldFieldMetaTypeName
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignResponse2]


class FieldCreateDocumentFieldFieldMetaNameResponse(BaseModel):
    type: FieldCreateDocumentFieldFieldMetaTypeName

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignResponse2],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldCreateDocumentFieldFieldMetaTypeInitials(str, Enum):
    INITIALS = "initials"


class FieldCreateDocumentFieldTextAlignResponse1(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldCreateDocumentFieldFieldMetaInitialsResponseTypedDict(TypedDict):
    type: FieldCreateDocumentFieldFieldMetaTypeInitials
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldCreateDocumentFieldTextAlignResponse1]


class FieldCreateDocumentFieldFieldMetaInitialsResponse(BaseModel):
    type: FieldCreateDocumentFieldFieldMetaTypeInitials

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldCreateDocumentFieldTextAlignResponse1],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldCreateDocumentFieldFieldMetaTypeSignature(str, Enum):
    SIGNATURE = "signature"


class FieldCreateDocumentFieldFieldMetaSignatureResponseTypedDict(TypedDict):
    type: FieldCreateDocumentFieldFieldMetaTypeSignature
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]


class FieldCreateDocumentFieldFieldMetaSignatureResponse(BaseModel):
    type: FieldCreateDocumentFieldFieldMetaTypeSignature

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12


FieldCreateDocumentFieldFieldMetaUnionTypedDict = TypeAliasType(
    "FieldCreateDocumentFieldFieldMetaUnionTypedDict",
    Union[
        FieldCreateDocumentFieldFieldMetaSignatureResponseTypedDict,
        FieldCreateDocumentFieldFieldMetaInitialsResponseTypedDict,
        FieldCreateDocumentFieldFieldMetaNameResponseTypedDict,
        FieldCreateDocumentFieldFieldMetaEmailResponseTypedDict,
        FieldCreateDocumentFieldFieldMetaDateResponseTypedDict,
        FieldCreateDocumentFieldFieldMetaRadioResponseTypedDict,
        FieldCreateDocumentFieldFieldMetaDropdownResponseTypedDict,
        FieldCreateDocumentFieldFieldMetaCheckboxResponseTypedDict,
        FieldCreateDocumentFieldFieldMetaTextResponseTypedDict,
        FieldCreateDocumentFieldFieldMetaNumberResponseTypedDict,
    ],
)


FieldCreateDocumentFieldFieldMetaUnion = TypeAliasType(
    "FieldCreateDocumentFieldFieldMetaUnion",
    Union[
        FieldCreateDocumentFieldFieldMetaSignatureResponse,
        FieldCreateDocumentFieldFieldMetaInitialsResponse,
        FieldCreateDocumentFieldFieldMetaNameResponse,
        FieldCreateDocumentFieldFieldMetaEmailResponse,
        FieldCreateDocumentFieldFieldMetaDateResponse,
        FieldCreateDocumentFieldFieldMetaRadioResponse,
        FieldCreateDocumentFieldFieldMetaDropdownResponse,
        FieldCreateDocumentFieldFieldMetaCheckboxResponse,
        FieldCreateDocumentFieldFieldMetaTextResponse,
        FieldCreateDocumentFieldFieldMetaNumberResponse,
    ],
)


class FieldCreateDocumentFieldResponseTypedDict(TypedDict):
    r"""Successful response"""

    envelope_id: str
    envelope_item_id: str
    type: FieldCreateDocumentFieldTypeResponse
    id: float
    secondary_id: str
    recipient_id: float
    page: float
    position_x: Any
    position_y: Any
    width: Any
    height: Any
    custom_text: str
    inserted: bool
    field_meta: Nullable[FieldCreateDocumentFieldFieldMetaUnionTypedDict]
    document_id: NotRequired[Nullable[float]]
    template_id: NotRequired[Nullable[float]]


class FieldCreateDocumentFieldResponse(BaseModel):
    r"""Successful response"""

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]

    type: FieldCreateDocumentFieldTypeResponse

    id: float

    secondary_id: Annotated[str, pydantic.Field(alias="secondaryId")]

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page: float

    position_x: Annotated[Any, pydantic.Field(alias="positionX")]

    position_y: Annotated[Any, pydantic.Field(alias="positionY")]

    width: Any

    height: Any

    custom_text: Annotated[str, pydantic.Field(alias="customText")]

    inserted: bool

    field_meta: Annotated[
        Nullable[FieldCreateDocumentFieldFieldMetaUnion],
        pydantic.Field(alias="fieldMeta"),
    ]

    document_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="documentId")
    ] = UNSET

    template_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="templateId")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["documentId", "templateId"]
        nullable_fields = ["fieldMeta", "documentId", "templateId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from documenso_sdk import models, utils
from documenso_sdk._hooks import HookContext
from documenso_sdk.types import OptionalNullable, UNSET
from documenso_sdk.utils import get_security_from_env
from documenso_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class TemplatesRecipients(BaseSDK):
    def get(
        self,
        *,
        recipient_id: float,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientGetTemplateRecipientResponse:
        r"""Get template recipient

        Returns a single recipient. If you want to retrieve all the recipients for a template, use the \"Get Template\" endpoint.

        :param recipient_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientGetTemplateRecipientRequest(
            recipient_id=recipient_id,
        )

        req = self._build_request(
            method="GET",
            path="/template/recipient/{recipientId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-getTemplateRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientGetTemplateRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetTemplateRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientGetTemplateRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetTemplateRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientGetTemplateRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetTemplateRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientGetTemplateRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetTemplateRecipientNotFoundErrorData, http_res
            )
            raise models.RecipientGetTemplateRecipientNotFoundError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetTemplateRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientGetTemplateRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        recipient_id: float,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientGetTemplateRecipientResponse:
        r"""Get template recipient

        Returns a single recipient. If you want to retrieve all the recipients for a template, use the \"Get Template\" endpoint.

        :param recipient_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientGetTemplateRecipientRequest(
            recipient_id=recipient_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/template/recipient/{recipientId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-getTemplateRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientGetTemplateRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetTemplateRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientGetTemplateRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetTemplateRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientGetTemplateRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetTemplateRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientGetTemplateRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetTemplateRecipientNotFoundErrorData, http_res
            )
            raise models.RecipientGetTemplateRecipientNotFoundError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetTemplateRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientGetTemplateRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        template_id: float,
        recipient: Union[
            models.RecipientCreateTemplateRecipientRecipient,
            models.RecipientCreateTemplateRecipientRecipientTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientCreateTemplateRecipientResponse:
        r"""Create template recipient

        Create a single recipient for a template.

        :param template_id:
        :param recipient:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientCreateTemplateRecipientRequest(
            template_id=template_id,
            recipient=utils.get_pydantic_model(
                recipient, models.RecipientCreateTemplateRecipientRecipient
            ),
        )

        req = self._build_request(
            method="POST",
            path="/template/recipient/create",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientCreateTemplateRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-createTemplateRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientCreateTemplateRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        template_id: float,
        recipient: Union[
            models.RecipientCreateTemplateRecipientRecipient,
            models.RecipientCreateTemplateRecipientRecipientTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientCreateTemplateRecipientResponse:
        r"""Create template recipient

        Create a single recipient for a template.

        :param template_id:
        :param recipient:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientCreateTemplateRecipientRequest(
            template_id=template_id,
            recipient=utils.get_pydantic_model(
                recipient, models.RecipientCreateTemplateRecipientRecipient
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/template/recipient/create",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientCreateTemplateRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-createTemplateRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientCreateTemplateRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def create_many(
        self,
        *,
        template_id: float,
        recipients: Union[
            List[models.RecipientCreateTemplateRecipientsRecipientRequest],
            List[models.RecipientCreateTemplateRecipientsRecipientRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientCreateTemplateRecipientsResponse:
        r"""Create template recipients

        Create multiple recipients for a template.

        :param template_id:
        :param recipients:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientCreateTemplateRecipientsRequest(
            template_id=template_id,
            recipients=utils.get_pydantic_model(
                recipients,
                List[models.RecipientCreateTemplateRecipientsRecipientRequest],
            ),
        )

        req = self._build_request(
            method="POST",
            path="/template/recipient/create-many",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientCreateTemplateRecipientsRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-createTemplateRecipients",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientCreateTemplateRecipientsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientsBadRequestErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientsBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientsUnauthorizedErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientsUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientsForbiddenErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientsForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientsInternalServerErrorData,
                http_res,
            )
            raise models.RecipientCreateTemplateRecipientsInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_many_async(
        self,
        *,
        template_id: float,
        recipients: Union[
            List[models.RecipientCreateTemplateRecipientsRecipientRequest],
            List[models.RecipientCreateTemplateRecipientsRecipientRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientCreateTemplateRecipientsResponse:
        r"""Create template recipients

        Create multiple recipients for a template.

        :param template_id:
        :param recipients:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientCreateTemplateRecipientsRequest(
            template_id=template_id,
            recipients=utils.get_pydantic_model(
                recipients,
                List[models.RecipientCreateTemplateRecipientsRecipientRequest],
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/template/recipient/create-many",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientCreateTemplateRecipientsRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-createTemplateRecipients",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientCreateTemplateRecipientsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientsBadRequestErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientsBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientsUnauthorizedErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientsUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientsForbiddenErrorData, http_res
            )
            raise models.RecipientCreateTemplateRecipientsForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateTemplateRecipientsInternalServerErrorData,
                http_res,
            )
            raise models.RecipientCreateTemplateRecipientsInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        template_id: float,
        recipient: Union[
            models.RecipientUpdateTemplateRecipientRecipient,
            models.RecipientUpdateTemplateRecipientRecipientTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientUpdateTemplateRecipientResponse:
        r"""Update template recipient

        Update a single recipient for a template.

        :param template_id:
        :param recipient:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientUpdateTemplateRecipientRequest(
            template_id=template_id,
            recipient=utils.get_pydantic_model(
                recipient, models.RecipientUpdateTemplateRecipientRecipient
            ),
        )

        req = self._build_request(
            method="POST",
            path="/template/recipient/update",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientUpdateTemplateRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-updateTemplateRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        template_id: float,
        recipient: Union[
            models.RecipientUpdateTemplateRecipientRecipient,
            models.RecipientUpdateTemplateRecipientRecipientTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientUpdateTemplateRecipientResponse:
        r"""Update template recipient

        Update a single recipient for a template.

        :param template_id:
        :param recipient:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientUpdateTemplateRecipientRequest(
            template_id=template_id,
            recipient=utils.get_pydantic_model(
                recipient, models.RecipientUpdateTemplateRecipientRecipient
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/template/recipient/update",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientUpdateTemplateRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-updateTemplateRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def update_many(
        self,
        *,
        template_id: float,
        recipients: Union[
            List[models.RecipientUpdateTemplateRecipientsRecipientRequest],
            List[models.RecipientUpdateTemplateRecipientsRecipientRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientUpdateTemplateRecipientsResponse:
        r"""Update template recipients

        Update multiple recipients for a template.

        :param template_id:
        :param recipients:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientUpdateTemplateRecipientsRequest(
            template_id=template_id,
            recipients=utils.get_pydantic_model(
                recipients,
                List[models.RecipientUpdateTemplateRecipientsRecipientRequest],
            ),
        )

        req = self._build_request(
            method="POST",
            path="/template/recipient/update-many",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientUpdateTemplateRecipientsRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-updateTemplateRecipients",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientsBadRequestErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientsBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientsUnauthorizedErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientsUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientsForbiddenErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientsForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientsInternalServerErrorData,
                http_res,
            )
            raise models.RecipientUpdateTemplateRecipientsInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def update_many_async(
        self,
        *,
        template_id: float,
        recipients: Union[
            List[models.RecipientUpdateTemplateRecipientsRecipientRequest],
            List[models.RecipientUpdateTemplateRecipientsRecipientRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientUpdateTemplateRecipientsResponse:
        r"""Update template recipients

        Update multiple recipients for a template.

        :param template_id:
        :param recipients:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientUpdateTemplateRecipientsRequest(
            template_id=template_id,
            recipients=utils.get_pydantic_model(
                recipients,
                List[models.RecipientUpdateTemplateRecipientsRecipientRequest],
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/template/recipient/update-many",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientUpdateTemplateRecipientsRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-updateTemplateRecipients",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientsBadRequestErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientsBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientsUnauthorizedErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientsUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientsForbiddenErrorData, http_res
            )
            raise models.RecipientUpdateTemplateRecipientsForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateTemplateRecipientsInternalServerErrorData,
                http_res,
            )
            raise models.RecipientUpdateTemplateRecipientsInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        recipient_id: float,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientDeleteTemplateRecipientResponse:
        r"""Delete template recipient

        :param recipient_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientDeleteTemplateRecipientRequest(
            recipient_id=recipient_id,
        )

        req = self._build_request(
            method="POST",
            path="/template/recipient/delete",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientDeleteTemplateRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-deleteTemplateRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientDeleteTemplateRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteTemplateRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientDeleteTemplateRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteTemplateRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientDeleteTemplateRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteTemplateRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientDeleteTemplateRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteTemplateRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientDeleteTemplateRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        recipient_id: float,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientDeleteTemplateRecipientResponse:
        r"""Delete template recipient

        :param recipient_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientDeleteTemplateRecipientRequest(
            recipient_id=recipient_id,
        )

        req = self._build_request_async(
            method="POST",
            path="/template/recipient/delete",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientDeleteTemplateRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-deleteTemplateRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientDeleteTemplateRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteTemplateRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientDeleteTemplateRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteTemplateRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientDeleteTemplateRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteTemplateRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientDeleteTemplateRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteTemplateRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientDeleteTemplateRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
from typing import List, Optional
from typing_extensions import TypedDict


class EnvelopeAttachmentDeleteRequestTypedDict(TypedDict):
    id: str


class EnvelopeAttachmentDeleteRequest(BaseModel):
    id: str


class EnvelopeAttachmentDeleteInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentDeleteInternalServerErrorIssue(BaseModel):
    message: str


class EnvelopeAttachmentDeleteInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentDeleteInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentDeleteInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: EnvelopeAttachmentDeleteInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentDeleteInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentDeleteForbiddenIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentDeleteForbiddenIssue(BaseModel):
    message: str


class EnvelopeAttachmentDeleteForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentDeleteForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentDeleteForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: EnvelopeAttachmentDeleteForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentDeleteForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentDeleteUnauthorizedIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentDeleteUnauthorizedIssue(BaseModel):
    message: str


class EnvelopeAttachmentDeleteUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentDeleteUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentDeleteUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: EnvelopeAttachmentDeleteUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentDeleteUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentDeleteBadRequestIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentDeleteBadRequestIssue(BaseModel):
    message: str


class EnvelopeAttachmentDeleteBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentDeleteBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentDeleteBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: EnvelopeAttachmentDeleteBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentDeleteBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentDeleteResponseTypedDict(TypedDict):
    r"""Successful response"""

    success: bool


class EnvelopeAttachmentDeleteResponse(BaseModel):
    r"""Successful response"""

    success: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, TypedDict


class DocumentRedistributeRequestTypedDict(TypedDict):
    document_id: float
    recipients: List[float]


class DocumentRedistributeRequest(BaseModel):
    document_id: Annotated[float, pydantic.Field(alias="documentId")]

    recipients: List[float]


class DocumentRedistributeInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class DocumentRedistributeInternalServerErrorIssue(BaseModel):
    message: str


class DocumentRedistributeInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentRedistributeInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentRedistributeInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: DocumentRedistributeInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentRedistributeInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentRedistributeForbiddenIssueTypedDict(TypedDict):
    message: str


class DocumentRedistributeForbiddenIssue(BaseModel):
    message: str


class DocumentRedistributeForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentRedistributeForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentRedistributeForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: DocumentRedistributeForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentRedistributeForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentRedistributeUnauthorizedIssueTypedDict(TypedDict):
    message: str


class DocumentRedistributeUnauthorizedIssue(BaseModel):
    message: str


class DocumentRedistributeUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentRedistributeUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentRedistributeUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: DocumentRedistributeUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentRedistributeUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentRedistributeBadRequestIssueTypedDict(TypedDict):
    message: str


class DocumentRedistributeBadRequestIssue(BaseModel):
    message: str


class DocumentRedistributeBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentRedistributeBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentRedistributeBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: DocumentRedistributeBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentRedistributeBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentRedistributeResponseTypedDict(TypedDict):
    r"""Successful response"""

    success: bool


class DocumentRedistributeResponse(BaseModel):
    r"""Successful response"""

    success: bool
